#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 18 10:50:06 2019
Data vizualisation
@author: juliengautier
"""
__version__='2022.09'
__author__='julien Gautier'


from visu import visual
from visu import visualLight

from visu.visualLight import SEELIGHT


from visu.visual import SEE
from visu.visual import runVisu
from visu import andor
from visu import WinCut
from visu import winMeas
from visu import winspec
from visu import winSuppE
from visu import winFFT
from visu import winZoom
from visu import winMath
from visu import winPointing
from visu import winHist
# try:
#     from visu import Win3D
# except :
#     print('')
# #from visu import moteurRSAI as RSAI
