#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 18 14:06:58 2019
open('README.md').read()
@author: juliengautier
"""

from setuptools import setup, find_packages
#import visu

setup(name='visu',version=2022.9,
      packages=find_packages(),
      author='Julien Gautier',
      author_email='julien.gautier@ensta.fr',
      description='Data visualization',
      include_package_data=True,
      package_data={'visu': ['incons/*.*']},
      classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Free for non-commercial use",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Visualization"
      ],
      url='https://github.com/julienGautier77/visu',
      install_requires=['pyqtgraph','matplotlib','PyQt6','qdarkstyle','scipy',])
